use std::ffi;
use std::os::raw;

type JitInit = extern fn(jit: *const Jit)
    -> *mut ffi::c_void;
type JitDeinit = extern fn(jit: *const Jit, plug: *mut ffi::c_void);
type JitRun = extern fn(jit: *const Jit, plug: *mut ffi::c_void);
type JitEvent = extern fn(jit: *const Jit, plug: *mut ffi::c_void, ev: *const JitEv);
type JitWork = extern fn(jit: *const Jit, plug: *mut ffi::c_void);

type JitAssertFunc = extern fn(data: *const ffi::c_void, condition: bool);
type JitPrintFunc = extern fn(data: *const ffi::c_void, fmt: *const raw::c_char);
type JitWorkWriteRequestFunc = extern fn(data: *const ffi::c_void, size: usize)
    -> *mut ffi::c_void;
type JitWorkWriteAdvanceFunc = extern fn(data: *const ffi::c_void, size: usize);
type JitWorkReadRequestFunc = extern fn(data: *const ffi::c_void, size: &mut usize)
    -> *const ffi::c_void;
type JitWorkReadAdvanceFunc = extern fn(data: *const ffi::c_void);

#[repr(C)]
pub struct JitAssert {
	pub data: *const ffi::c_void,
	pub func: JitAssertFunc
}

#[repr(C)]
pub struct JitPrint {
	pub data: *const ffi::c_void,
	pub func: JitPrintFunc
}

#[repr(C)]
pub struct JitWorkWriteRequest {
	pub data: *const ffi::c_void,
	pub func: JitWorkWriteRequestFunc
}

#[repr(C)]
pub struct JitWorkWriteAdvance {
	pub data: *const ffi::c_void,
	pub func: JitWorkWriteAdvanceFunc
}

#[repr(C)]
pub struct JitWorkReadRequest {
	pub data: *const ffi::c_void,
	pub func: JitWorkReadRequestFunc
}

#[repr(C)]
pub struct JitWorkReadAdvance {
	pub data: *const ffi::c_void,
	pub func: JitWorkReadAdvanceFunc
}

#[repr(C)]
pub struct Jit {
	pub mask: u32,

	pub srate: u32,
	pub nchannels: u32,
	pub ncontrols: u32,

	pub nsamples: u32,
	pub audio_in: *const *const f32,
	pub audio_out: *const *mut f32,
	pub control: *const f32,

	pub assert: JitAssert,
	pub print: JitPrint,
    pub work_write_request: JitWorkWriteRequest,
    pub work_write_advance: JitWorkWriteAdvance,
    pub work_read_request: JitWorkReadRequest,
    pub work_read_advance: JitWorkReadAdvance
}

#[repr(C)]
pub struct JitEv {
    pub etype:    u32,
	pub index:    u32,
	pub gain:     f32,
	pub freq:     f32,
	pub pressure: f32,
	pub timbre:   f32
}

#[repr(C)]
pub struct JitDesc {
	pub init:   JitInit,
	pub deinit: JitDeinit,
	pub run:	JitRun,
	pub event:	JitEvent,
	pub work:	JitWork
}
