/*
 * Copyright (c) 2019 Hanspeter Portner (dev@open-music-kontrollers.ch)
 *
 * This is free software: you can redistribute it and/or modify
 * it under the terms of the Artistic License 2.0 as published by
 * The Perl Foundation.
 *
 * This source is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * Artistic License 2.0 for more details.
 *
 * You should have received a copy of the Artistic License 2.0
 * along the source as a COPYING file. If not, obtain it from
 * http://www.perlfoundation.org/artistic_license_2_0.
 */

#ifndef _JIT_LV2_H
#define _JIT_LV2_H

#include <stdint.h>
#if !defined(_WIN32)
#	include <sys/mman.h>
#else
#	define mlock(...)
#	define munlock(...)
#endif

#include <lv2/lv2plug.in/ns/ext/atom/atom.h>
#include <lv2/lv2plug.in/ns/ext/atom/forge.h>
#include <lv2/lv2plug.in/ns/ext/urid/urid.h>
#include <lv2/lv2plug.in/ns/ext/time/time.h>
#include <lv2/lv2plug.in/ns/ext/midi/midi.h>
#include <lv2/lv2plug.in/ns/ext/state/state.h>
#include <lv2/lv2plug.in/ns/ext/worker/worker.h>
#include <lv2/lv2plug.in/ns/ext/log/log.h>
#include <lv2/lv2plug.in/ns/ext/log/logger.h>
#include <lv2/lv2plug.in/ns/ext/patch/patch.h>
#include <lv2/lv2plug.in/ns/ext/options/options.h>
#include <lv2/lv2plug.in/ns/ext/buf-size/buf-size.h>
#include <lv2/lv2plug.in/ns/extensions/ui/ui.h>
#include <lv2/lv2plug.in/ns/lv2core/lv2.h>

#define JIT_URI    "http://open-music-kontrollers.ch/lv2/jit"
#define JIT_PREFIX JIT_URI "#"

// plugin uris
#define JIT__mono            JIT_PREFIX "mono"
#define JIT__stereo          JIT_PREFIX "stereo"

// plugin UI uris
#define JIT__ui              JIT_PREFIX "ui"

// param uris
#define JIT__code            JIT_PREFIX "code"
#define JIT__error           JIT_PREFIX "error"
#define JIT__xfadeDuration   JIT_PREFIX "xfadeDuration"
#define JIT__fontHeight      JIT_PREFIX "fontHeight"
#define JIT__control_1       JIT_PREFIX "control_1"
#define JIT__control_2       JIT_PREFIX "control_2"
#define JIT__control_3       JIT_PREFIX "control_3"
#define JIT__control_4       JIT_PREFIX "control_4"
#define JIT__control_5       JIT_PREFIX "control_5"
#define JIT__control_6       JIT_PREFIX "control_6"
#define JIT__control_7       JIT_PREFIX "control_7"
#define JIT__control_8       JIT_PREFIX "control_8"
#define JIT__control_9       JIT_PREFIX "control_9"
#define JIT__control_10      JIT_PREFIX "control_10"
#define JIT__control_11      JIT_PREFIX "control_11"
#define JIT__control_12      JIT_PREFIX "control_12"
#define JIT__control_13      JIT_PREFIX "control_13"
#define JIT__control_14      JIT_PREFIX "control_14"
#define JIT__control_15      JIT_PREFIX "control_15"
#define JIT__control_16      JIT_PREFIX "control_16"

#define NCONTROLS 16
#define MAX_NPROPS (4 + NCONTROLS)
#define CODE_SIZE 0x10000 // 64 K
#define ERROR_SIZE 0x2000 // 8 K
#define BUF_SIZE (CODE_SIZE * 4)

#define CONTROL(NUM) \
{ \
	.property = JIT_PREFIX"control_"#NUM, \
	.offset = offsetof(plugstate_t, control) + (NUM-1)*sizeof(float), \
	.type = LV2_ATOM__Float \
}

typedef struct _plugstate_t plugstate_t;

struct _plugstate_t {
	char code [CODE_SIZE];
	char error [ERROR_SIZE];
	float control [NCONTROLS];
	int32_t xfade_dur;
	int32_t font_height;
};

#endif // _JIT_LV2_H
