#include <stddef.h>
#include <stdlib.h>

#include <jit_api.h>
#include <jit_util.h>

typedef struct _plug_t plug_t;

struct _plug_t {
	uint32_t count;
};

static void *
init(jit_t *const jit)
{
	jit_printf(jit, "[%s]", __func__);

	plug_t *plug = calloc(1, sizeof(plug_t));
	jit_assert(jit, plug);

	return plug;
}

static void
deinit(jit_t *const jit, void *const data)
{
	jit_printf(jit, "[%s]", __func__);

	plug_t *plug = data;
	jit_assert(jit, plug);

	free(plug);
}

static void
run(jit_t *const jit, void *const data)
{
	plug_t *plug = data;
	jit_assert(jit, plug);

	// count samples, print something every second
	plug->count += jit->nsamples;
	if(plug->count >= jit->srate)
	{
		jit_printf(jit, "[%s]", __func__);

		plug->count -= jit->srate;
	}

	for(uint32_t n = 0; n < jit->nchannels; n++)
	{
		for(uint32_t i = 0; i < jit->nsamples; i++)
		{
			jit->audio_out[n][i] = jit->control[0] * jit->audio_in[n][i];
		}
	}
}

JIT_EXPORT jit_desc_t const jit_desc = {
	.init = init,
	.deinit = deinit,
	.run = run
};

// vim: set syntax=c:
// jit: gcc -std=c11 -ffast-math -fvisibility=hidden -O3 -shared -fpic -fPIC -Wall -Wextra -Wpedantic -Wl,-z,defs -I %b -o %o -xc %i -lm 2>&1:
