#include <stddef.h>
#include <stdlib.h>

#include <jit_api.h>
#include <jit_util.h>

static float
_drain_val(jit_t *const jit)
{
	float const *val;
	size_t len;
	float gain = 0.f;

	while( (val = jit_work_read_request(jit, &len)) )
	{
		gain = *val;

		jit_work_read_advance(jit);
	}

	return gain;
}

static void
_wake_worker(jit_t *const jit)
{
	if(jit_work_write_request(jit, 0))
	{
		jit_work_write_advance(jit, 0);
	}
}

static void
run(jit_t *const jit, void *const data JIT_UNUSED)
{
	float const gain = _drain_val(jit);
	_wake_worker(jit);

	for(uint32_t n = 0; n < jit->nchannels; n++)
	{
		for(uint32_t i = 0; i < jit->nsamples; i++)
		{
			jit->audio_out[n][i] = gain * jit->audio_in[n][i];
		}
	}
}

static void
_drain_dummy(jit_t *const jit)
{
	size_t len;

	while(jit_work_read_request(jit, &len))
	{
		jit_work_read_advance(jit);
	}
}

static void
_produce_val(jit_t *const jit)
{
	float *val;
	const size_t len = sizeof(float);

	if( (val = jit_work_write_request(jit, len)) )
	{
		*val = (float)rand() / RAND_MAX;

		jit_work_write_advance(jit, len);
	}
}

static void
work(jit_t *const jit, void *const data JIT_UNUSED)
{
	_drain_dummy(jit);
	_produce_val(jit);
}

JIT_EXPORT jit_desc_t const jit_desc = {
	.run = run,
	.work = work
};

// vim: set syntax=c:
// jit: gcc -std=c11 -ffast-math -fvisibility=hidden -O3 -shared -fpic -fPIC -Wall -Wextra -Wpedantic -Wl,-z,defs -I %b -o %o -xc %i -lm 2>&1:
